package de.inovex.academy.hadoop.candy;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mrunit.mapreduce.MapDriver;
import org.apache.hadoop.mrunit.mapreduce.MapReduceDriver;
import org.apache.hadoop.mrunit.mapreduce.ReduceDriver;
import org.junit.Before;
import org.junit.Test;

import de.inovex.academy.hadoop.util.IntSumReducer;

public class CandyCountTest {
    MapReduceDriver<LongWritable, Text, Text, IntWritable, Text, IntWritable> mapReduceDriver;
    MapDriver<LongWritable, Text, Text, IntWritable> mapDriver;
    ReduceDriver<Text, IntWritable, Text, IntWritable> reduceDriver;

  @Before
      public void setUp() {
      Mapper<LongWritable, Text, Text, IntWritable> mapper = 
	  new ColorMapper();
      Reducer<Text, IntWritable, Text, IntWritable> reducer = 
	  new IntSumReducer();
      mapDriver = new MapDriver<LongWritable, Text, Text, IntWritable>();
      mapDriver.setMapper(mapper);
      reduceDriver = new ReduceDriver<Text, IntWritable, Text, IntWritable>();
      reduceDriver.setReducer(reducer);
      mapReduceDriver = new MapReduceDriver<LongWritable, Text, Text, IntWritable, Text, IntWritable>();
      mapReduceDriver.setMapper(mapper);
      mapReduceDriver.setReducer(reducer);
  }

  @Test
      public void testMapperHappy() throws IOException {
      mapDriver.withInput(new LongWritable(1), new Text("red\tmilk\tS\t9\t30\t171"));
      mapDriver.withOutput(new Text("red"), new IntWritable( 1 ));

      mapDriver.runTest();
  }

  @Test
      public void testReducer() throws IOException {
      List<IntWritable> values = new ArrayList<IntWritable>();
      values.add(new IntWritable(5));
      values.add(new IntWritable(6));
      values.add(new IntWritable(10));
      reduceDriver.withInput(new Text("red"), values);
      reduceDriver.withOutput(new Text("red"), new IntWritable( 21 ));
      reduceDriver.runTest();
  }

  @Test
      public void testMapReduce() throws IOException {
      mapReduceDriver.withInput(new LongWritable(1), new Text("crushed\tcrushed\tcrushed\t-1\t-1\t-1"));
      mapReduceDriver.addOutput(new Text("crushed"), new IntWritable( 1 ));
      mapReduceDriver.runTest();
  }

}
